=begin
Copyright 2015 The MITRE Corporation

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.


A replacement for MedAttrib.pl. 

The MedAttrib class method parse(text) returns an
AnnotatedString which contains the original text and a list of the tags
found. Because "doseamount" depends on the presence of other tags,
the taggers have been split into pre- and post-doseamount taggers.

Example usage:
tagger = MedAttrib.new
annotated_string = tagger.parse("take daily.")
# annotated_string is now "take daily." with a "freq" tag labeling "daily" in standoff annotation (class AnnotatedString). This can be printed as inline xml using AnnotatedString#to_s:

puts annotated_string
#=> "take <freq>daily</freq>."
=end

require 'standoff'

require_relative 'tagger.rb' #taggers based on regex contexts
require_relative 'list_tagger.rb' #taggers based on term lists

#Tagger child classes:
require_relative 'taggers/dose_amount_tagger.rb'
require_relative 'taggers/strength_tagger.rb'
require_relative 'taggers/strength_unit_tagger.rb'
require_relative 'taggers/indication_tagger.rb'
require_relative 'taggers/duration_tagger.rb'
require_relative 'taggers/dispense_quantity_tagger.rb'
require_relative 'taggers/refill_tagger.rb'
require_relative 'taggers/substitution_tagger.rb'
require_relative 'taggers/timing_tagger.rb'

#ListTagger child classes:
require_relative 'list_taggers/dose_form_tagger.rb'
require_relative 'list_taggers/dose_amount_list_tagger.rb'
require_relative 'list_taggers/route_tagger.rb'
require_relative 'list_taggers/frequency_tagger.rb'


class MedAttrib
  def parse (text)

#    text = REXML::Text.new(text, true, nil, false).to_s # escape special characters
    taggersExceptDoseAmount = [StrengthTagger.new,
                StrengthUnitTagger.new,
                DoseFormTagger.new,
                DoseAmountListTagger.new,
                RouteTagger.new,
                IndicationTagger.new,
                FrequencyTagger.new,
                TimingTagger.new,
                SubstitutionTagger.new,
                RefillTagger.new,
                DurationTagger.new,
                DispenseQuantityTagger.new]
               
    tags = []
    taggersExceptDoseAmount.each {|tagger| tags += tagger.parse_main text}
    as = Standoff::AnnotatedString.new(:signal => text, :tags => tags)
    DoseAmountTagger.new.parse_annotated_string as
    
    partial_xml = as.to_s
    as

  end

end
