=begin
Copyright 2015 The MITRE Corporation

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
=end

class RefillTagger < Tagger
    def initialize
        @default_precondition = /\d+\s+refills?/i
        @default_search = /(\d+)(\s+refills?)/i
        @content_group = 1
        @name = 'refill'
    end
    def parse_text(text, precondition, search, content_group)
        tags = super text, @default_precondition, @default_search, @content_group

        local_precondition = /refill\s+\d+\s+times?/i
        local_search = /(refill\s+)(\d+)(\s+times?)/i
        local_content_group = 2
        tags += super text, local_precondition, local_search, local_content_group

        @precondition = /refills?\s+times\s+\d+/i
        @search = /(refills?\s+times\s+)(\d+)/i
        tags += super text, local_precondition, local_search, local_content_group

        local_precondition = /(?x)\b(?:no|zero|one|two|three|four|five|six|seven|
                   eight|nine|ten|eleven|twelve)\s+refills?/i
        local_search = /(?x)\b(no|zero|one|two|three|four|five|six|seven|
                       eight|nine|ten|eleven|twelve)(\s+refills?)/i
        local_content_group = 1
        tags += super text, local_precondition, local_search, local_content_group

        @precondition = /(?x)refill\s+(?:no|zero|one|two|three|four|five|six|
                    seven|eight|nine|ten|eleven|twelve)\s+times?/i
        @search = /(?x)(refill\s+)(no|zero|one|two|three|four|five|six|
                       seven|eight|nine|ten|eleven|twelve)(\s+times?)/i
        local_content_group = 2
        tags += super text, local_precondition, local_search, local_content_group
        
        @precondition = /(?x)(?:refills?\s+(?:times\s+)?)(?:none|zero|one|two|
                   three|four|five|six|seven|eight|nine|ten|eleven|twelve)+/i
        @search = /(?x)(refills?\s+(?:times\s+)?)(none|zero|one|two|
                    three|four|five|six|seven|eight|nine|ten|eleven|twelve)/i
        
        tags += super text, local_precondition, local_search, local_content_group
        return tags
    end
end
